# `syscall` for node.js

Execute POSIX/Linux `syscall` command from Node.js.

Usage, see [syscall reference](https://filippo.io/linux-syscall-table/):

```js
var syscall = require('libsys').syscall;

// Print `Hello world` in console using kernel's `syscall` command.
var buf = new Buffer('Hello world\n');
syscall(1, 1, buf, buf.length);

// Where, on x86_64 Linux:
// 1 - `sys_write` system call
// 1 - STDOUT file descriptor
// buf - pointer to data in memory
// buf.length - size of the memory block to print
```

See [libjs](http://www.npmjs.com/package/libjs) for POSIX command implementation.

Part of [jskernel](http://www.github.com/streamich/jskernel) project.

## Reference

### `syscall` and `syscall64`

```ts
type TArg = number|string|ArrayBuffer|Uint8Array|Buffer;
syscall(command: number, ...args: TArg[]): number;
syscall64(command: number, ...args: TArg[]): [number, number];
```

`syscall` accepts up to 6 command arguments `args`, which are treated as
follows depending on their type:

 - `number` is put directly in the appropriate CPU register as is.
 - `string` gets converted to *C*'s `char *` string and that pointer is used as an argument.
 - for `ArrayBuffer`, `Uint8Array`, and `Buffer` pointer to the raw data in-memory is used as argument.
    
`syscall` returns a `number` which is the result returned by the kernel,
negative numbers usually represent an error. If sytem call returns -1, the
C `errno` variable will be returned, which usually has more information about the error.

### `malloc` as `malloc64`

Returns a `Buffer` object of size `size` that is mapped to memory location
specified in `addr` argument.

```ts
malloc(addr: number, size: number): ArrayBuffer;
malloc(lo: number, hi: number, size: number): ArrayBuffer;
```

### `addressBuffer` and `addressBuffer64`

Return memory address of `Buffer`'s data contents.

```ts
addressBuffer(buffer: Buffer): number;
addressBuffer64(buffer: Buffer): [number, number];
```
    
`addr64` returns a tuple which represents 64-bit number, where first element contains the lower
32 bits and second element has the high 32 bits.

### `addressArrayBuffer` and `addressArrayBuffer64`

Returns `ArrayBuffer` address.

### `addressUint8Array` and `addressUint8Array64`

Returns `Uint8Array` address.

### `call` as `call64`

Execute machine code at specified memory address. 

```ts
export function call(addr: number);
export function call64(addr_lo: number, addr_hi: number);
```
    
### `errno`

Returns `errno` variable.

```ts
function errno(): number;
```
    
## Installation

    npm i libsys
    
Compiles on Ubuntu 14.04 x86_64 running under Docker with Node.js 4.4.3, has not
been tested on other machines.
    
## Building addon:
    
    node-gyp configure
    node-gyp rebuild
